import dayjs from 'dayjs'
import { Toast } from 'vant'

export const uploadIdcard = api => {
  // 表格数据
  const formData = {}
  // 是否ocr识别错误
  let error = void 0
  // 返回对象
  return async dataURLs => {
    await Promise.all(
      dataURLs.map(dataURL => {
        return new Promise(async(resolve) => {
          try {
            // 如果先压缩图片
            const ocrPic = dataURL.replace('data:image/jpeg;base64,', '').replace('data:image/png;base64,', '')
            // 文件可访问路径和文件的uuid
            const respData = await api({ ocrPic })
            // 身份证信息
            if (respData.side === 'back') {
              formData.fullName = respData.fullName
              formData.gender = respData.gender === '男' ? 1 : 2
              formData.certNumber = respData.certNumber
              formData.certAddress = respData.address
            } else {
              // 过期时间格式化
              const validDate = respData.validDate.split('-')
              // 如果当前是长期
              if (validDate.length === 2) {
                formData.certEffectiveDate = validDate[0].replace(/\./g, '-')
                formData.certExpirationDate = validDate[1].replace(/\./g, '-')
                // 当前过期时间
                const expireDateInDayjs = dayjs(formData.certExpirationDate, 'YYYY-MM-DD')
                // 如果身份证读取的过期时间小于当前天数
                if (expireDateInDayjs.isBefore(dayjs().startOf('date'))) {
                  Toast('当前身份证已过期，请注意更换')
                  formData.certExpirationDate = ''
                }
              } else {
                formData.certExpirationDate = respData.validDate
              }
            }
          } catch (e) {
            error = [1011, 1809].includes(e.code) ? 'detect error' : 'system error'
          }
          resolve()
        })
      })
    )
    return { error, data: { ...formData }}
  }
}
