/**
 * 如果当前是必填项
 * @param value 当前输入的值
 */
const isRequired = value => {
  if (!value && value !== 0) {
    return '请输入内容'
  }
}

/**
 * 如果当前是身份证号
 * @param value 当前输入的值
 */
export const isIdcard = value => {
  if (`${value}`.length !== 18) {
    return '请输入正确的身份证号码'
  }
}

/**
 * 如果当前是VIN码
 * @param value 当前输入的值
 */
export const isVin = value => {
  if (`${value}`.length !== 17) {
    return '请输入正确的VIN码'
  }
}

export const isIccid = value => {
  if (`${value}`.length !== 20) {
    return '请输入正确的ICCID'
  }
}

/**
 * 是否是身份证号
 * @param value 当前输入的值
 */
const isIdcardPic = value => {
  if (!value || value.length === 0 || value.length !== value.filter(v => !!v).length) {
    return '请上传证件照片'
  }
  // 如果上传照片小于2张
  if (value.length < 2) {
    return '请至少上传2张证件照片'
  }
}

/**
 * 当前是否是照片
 * @param value 当前输入的值
 */
const isEmptyPics = value => {
  if (!value || value.length === 0) {
    return '请上传文件'
  }
}

/**
 * 当前是否是正确的手机号
 * @param value 当前输入的值
 */
const isPhone = value => {
  if (!`${value}`.match(/^1\d{10}$/)) {
    return '请输入正确的手机号'
  }
}

// 等待校验的规则
const rules = {
  certType: [isRequired],
  fullName: [isRequired],
  gender: [isRequired],
  certAddress: [isRequired],
  certExpirationDate: [isRequired],
  certNumber: [isRequired],
  certPic: [isIdcardPic],
  contractPic: [isEmptyPics],
  purchaseContractPic: [isEmptyPics],
  transferCertificatePic: [isEmptyPics],
  authorizationLetterPic: [isEmptyPics],
  vin: [isRequired, isVin],
  iccid: [isRequired, isIccid],
  phone: [isRequired, isPhone],
  companyName: [isRequired],
  companyCertNumber: [isRequired],
  companyCertAddress: [isRequired],
  companyContactAddress: [isRequired]
}

/**
 * 校验用户输入
 * @param  校验参数
 */
export function validate(values, ruleKeys = ['fullName', 'gender', 'certType', 'certNumber', 'certAddress', 'certExpirationDate']) {
  // 校验位置的索引
  let index = 0
  // 校验结果
  let result = true
  // 校验失败字段
  let failField = ''
  // 错误描述
  let failMessage = ''
  // 如果当前没有遍历完全
  while (index < ruleKeys.length && result) {
    // 当前需要校验的key
    const ruleKey = ruleKeys[index]
    // 如果当前配置了规则
    if (rules[ruleKey]) {
      rules[ruleKey].forEach(valid => {
        // 获取当前校验结果
        failMessage = failMessage || valid(values[ruleKey])
        // 如果当前校验失败
        result = result && !failMessage
      })
      // 如果当前校验失败
      if (!result) {
        failField = ruleKey
      }
    }
    // 索引累加
    index++
  }
  return [failField, failMessage]
}

